SystemOrganization addCategory: #'System-CommandLine'!
SystemOrganization addCategory: #'System-CommandLine-Test'!
SystemOrganization addCategory: #'System-CommandLine-Tests'!

Object subclass: #CommandLineArguments
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments commentStamp: 'StephaneDucasse 3/14/2011 14:14' prior: 0!
CommandLine represents the arguments given on the CommandLine.
CommandLine is a singleton, so it can not be instantiated.

If you run pharo by:
	pharo a b c d e
then, at start up, the unique instance of CommandLine will be created, and will receive #(a b c d e) as parameter for its input: message.

---
All blocks you register on this class will be evaluated at the start up.
Please note that CommandLine DO NOT REPLACE the start up list (in SmalltalkImage).

Finally even if the condition block is always true, it does not mean that the action block will be executed; for example, when there is no
 parameter on the command line, it will not be. If there is no command line argument the following will not be executed. 

	CommandLine register: [ :commandLine | "anything"] if: [ :commandLine | true ]  name: #smth.!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:01'!
uniqueInstance
	^ singleton ifNil: [singleton := self new initialize]! !

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:00'!
withArguments: aCollection
	^ self basicNew 
		initializeWithArguments: aCollection;
		yourself! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 19:57'!
argumentAt: index
	^ arguments at: index ! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:42'!
arguments
	^ arguments! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:49'!
commandLineArguments
	"self commandLineArguments"
	
	| documentPath args |
	
	args := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ args addFirst: documentPath ].
 	^ args.! !

!CommandLineArguments methodsFor: 'copying' stamp: 'CamilloBruni 5/1/2012 20:07'!
copySubcommand
	"return a new copy of this CommandLine without the first arguments"
	^ self class withArguments: arguments allButFirst! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:50'!
hasArguments
	^ arguments size > 0! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:49'!
hasFileWithExtension: aFileExtension
	"return true if the first argument has the given file extension"
	self withFirstArgument: [ :arg| 
		^ arg endsWith: aFileExtension ].
	^ false! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 21:21'!
initialize
	
	| documentPath  |
	
	arguments := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ arguments addFirst: documentPath ].! !

!CommandLineArguments methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/1/2012 20:01'!
initializeWithArguments: aCollection
	super initialize.
	arguments := aCollection.! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 4/28/2012 19:54'!
withFirstArgument: aBlock
	self arguments ifEmpty: [ ^ self ].
	^ aBlock value: self arguments first! !

Object subclass: #CommandLineHandler
	instanceVariableNames: 'commandLine stdout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!CommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler is activated by the CommandLine. 

The responsible handler with the highest priority is selected and its instance-side method #activate is invoked.

By default the handlers are selected by their class name. In the following shell invocation the FooHandler is chosen:

	$PATH_TO_VM myImage.image FooHandler

A handler may provide a short name with the class-side #commandName method. If the FooHandler defined #commandName returning 'foo' it would be activated with the following shell invocation:
	
	$PATH_TO_VM myImage.image foo
	
For more sophisticated handler selection the CommandLineHandler should implement the #isResponsibleFor: class-side method. An instance of the current command line options is passed to this method which should then return a boolean.

Between all the responsible handlers the one with the highes #priority is chosen. To change the priority overwrite the class-side accessor.
!

CommandLineHandler subclass: #BasicCodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!BasicCodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!BasicCodeLoader class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/1/2012 20:48'!
isResponsibleFor: aCommandLine
	(aCommandLine hasFileWithExtension: '.st')
		ifTrue: [ ^ true ].
	^ super isResponsibleFor: aCommandLine! !

!BasicCodeLoader methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 22:04'!
activate
	self loadSourceFiles: self arguments.
	self installSourceFiles.! !

!BasicCodeLoader methodsFor: 'private' stamp: 'CamilloBruni 4/28/2012 21:48'!
handleError: error	
	
	"for syntax errors we can used the default action"	"otherwise resignal it"
	(error isKindOf: SyntaxErrorNotification)
		ifTrue: [ error defaultAction ]
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'private' stamp: 'EstebanLorenzano 4/4/2012 10:38'!
handleError: error reference: aReference
	"Print a header before failing on an errro / syntax notification from the the script loaded by the given request"
	|stderr|
	
	"spit out a warning if in headless mode, otherwise a debugger will popup"
	Smalltalk isHeadless 
		ifTrue: [ self inform: 'Errors in script loaded from ', aReference fullName ].
																														
	(error isKindOf: SyntaxErrorNotification)
		"for syntax errors we can used the default action"
		ifTrue: [ error defaultAction ]
		"otherwise resignal it"
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:39'!
installSourceFile: aReference
	"Install the previously loaded source file"

	aReference readStreamDo: [ :stream | 			
		[ ^ (FileSystemFileStream with: stream contents asString) reset fileIn ]
			on: Error, ParserNotification
			do: [ :e| 	self handleError: e reference: aReference ] ].! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:31'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles ifNil: [ ^self ].
	[ sourceFiles do: [ :reference | self installSourceFile: reference ] ]
	ensure: [ sourceFiles := nil ].
	! !

!BasicCodeLoader methodsFor: 'loading' stamp: 'CamilloBruni 5/1/2012 20:51'!
loadSourceFiles: anArray
	"Load all the source files in the given array."

	sourceFiles := anArray collect: [ :each | 
						(FileSystem disk resolve: each) asFileReference ].
! !

BasicCodeLoader subclass: #EvaluateCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!EvaluateCommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler that reads a string from the command line and outputs the evaluated result.!

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 22:48'!
commandName
	^ 'eval'! !

!EvaluateCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 22:54'!
isResponsibleFor: aCommandLine
	aCommandLine withFirstArgument: [ :arg| 
		(#('-e' '--evaluate') includes: arg)
			ifTrue: [ ^ true ]].
	^ super isResponsibleFor: aCommandLine! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 20:07'!
activate
	self evaluate: self arguments first.
	self quit.! !

!EvaluateCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:50'!
evaluate: sourceString
		
	[ self stdout print: (Compiler evaluate: sourceString); cr ]
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e  ].! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:08'!
activateWith: aCommandLine 
	
	^ self new 
		commandLine: (self prepareSubCommandLine: aCommandLine);
		activate! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:38'!
allHandlers
	^ self allSubclasses reject: [ :handler| handler isAbstract ]! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:56'!
commandName
	" Overwrite this method to provide a short name for this command handler.
	You may pass the commandName as first argument to the image to select this handler"
	^ self name! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:53'!
description
	"This method should return a short one-line description of the command"
	^ nil! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:39'!
isAbstract
	^ self = CommandLineHandler! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 23:08'!
isResponsibleFor: aCommandLine
	
	aCommandLine withFirstArgument: [ :arg| 
		^ arg = self name or: [ arg = self commandName ]].
	
	^ false! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:08'!
prepareSubCommandLine: aCommandLine
	
	aCommandLine withFirstArgument: [ :arg|
		arg = self commandName 
			ifTrue: [ ^ aCommandLine copySubcommand ]].
	
	^ aCommandLine! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:51'!
priority
	^ 0! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/29/2012 17:37'!
selectHandlersFor: aCommandLine
	^ self allHandlers select: [ :handlerClass|
		handlerClass isResponsibleFor: aCommandLine ]! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:33'!
<< aString
	^ self stdout 
		nextPutAll: aString;
		yourself! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 19:46'!
activate
	 self subclassResponsibility! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:43'!
allHandlers 
	^ CommandLineHandler allHandlers ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:56'!
arguments
	^ commandLine arguments ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine
	^ commandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine: aCommandLine 
	commandLine := aCommandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:42'!
commandName
	^ self class commandName! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:52'!
description
	^ self class description! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitFailure 
	^ Smalltalk exitFailure! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitSucess
	^ Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initialize
	super initialize.
	self initializeStdout
	! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initializeStdout
	"install the line end conversion and initialize the converter"
	FileStream stdout wantsLineEndConversion: true; converter.
	stdout := FileStream stdout.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 21:36'!
quit
	Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 19:47'!
stdout
	^ stdout! !

CommandLineHandler subclass: #DefaultCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!DefaultCommandLineHandler commentStamp: '<historical>' prior: 0!
A DefaultCommandLineHandler handles default command line arguments and options.

The DefaultCommandLineHandler is activated before all other handlers. It first checks if another handler is available. If so it will activate the found handler.!

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/1/2012 13:54'!
description
	^ 'default command line handler'! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:12'!
initialize 
	Smalltalk addToStartUpList: self.! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:27'!
isResponsibleFor: aCommandLine
	^ true! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 21:41'!
priority 
	"lowest priority"
	^ -1! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:18'!
startUp: resuming

	"only handle when lauching a new image"
	resuming ifFalse: [ ^ self ].
	Smalltalk addDeferredStartupAction: [ self new activate ]! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 21:23'!
activate
	self arguments ifEmpty: [ ^ self ].
	
	^ self handleArgument: self arguments first.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:54'!
copyright
	self stdout 
		nextPutAll: Smalltalk licenseString; cr.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 5/2/2012 00:23'!
error
	self arguments size = 1
		ifTrue:  [
			(self arguments first beginsWith: '-')
				ifTrue:  [ self << 'Invalid option: ']
				ifFalse: [ self << 'Invalid sub command: ']]
		ifFalse: [ self << 'Invalid arguments: "'].
		
	(self << (' ' join: self arguments)) 
		nextPutAll: '"'; cr.
		
	self printHelp.
	
	Smalltalk isInteractive ifFalse: [ ^ self exitFailure ].
	Error signal: 'Invalid arguments'.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleArgument: aString
	
	"give priority to subcommands"
	self handleSubcommand == self
		ifFalse: [ ^ self ].
		
	"check for default options"
	aString = '--version'
		ifTrue: [ ^ self version ].
		
	aString = '--help'
		ifTrue: [ ^ self help ].
		
	aString = '--list'
		ifTrue: [ ^ self list ].
		
	aString = '--copyright'
		ifTrue: [ ^ self copyright ].
	
	"none of the previous options matched hence we output an error message"
	self error.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleSubcommand
	"check if there is an active subcommand"
	
	| handlers |
	
	handlers := self selectHandlers.
	
	"ignore the default command line handler"
	handlers first = DefaultCommandLineHandler
		ifTrue: [ ^ self ].
		
	^ handlers first activateWith: commandLine.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/29/2012 16:20'!
help
	self printHelp.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 21:24'!
initialize
	super initialize.
	commandLine := CommandLineArguments new! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 20:09'!
list
	| shortNames maxShortNameSize |
	self stdout 
		nextPutAll: 'Currently installed Command Line Handlers:'; cr.
		
	shortNames := self allHandlers collect: #commandName.
	maxShortNameSize := (shortNames detectMax: [ :name|  name size ]) size.
	
	self allHandlers do: [ :handler|
		self stdout 
			nextPutAll: '    ';
			nextPutAll: (handler commandName 
								padded: #right to: maxShortNameSize with: $ ); 
			nextPutAll: '    ';
			nextPutAll: (handler description ifNil: [ '--']);cr].
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 13:38'!
printHelp
	self stdout 
		nextPutAll: 'Usage:'; cr;
		nextPutAll: '    --help       print this help message'; cr;
		nextPutAll: '    --copyright  print the copyrights'; cr;
		nextPutAll: '    --version    print the version for the image and the vm'; cr;
		nextPutAll: '    --list       list a description of all active command line handlers'; cr.! !

!DefaultCommandLineHandler methodsFor: 'private' stamp: 'CamilloBruni 5/1/2012 21:22'!
selectHandlers
	| handlers |
	handlers := CommandLineHandler selectHandlersFor: commandLine.
	handlers := handlers sort: [ :a :b| a priority >= b priority ].
	^ handlers! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 22:00'!
version
	self stdout 
		nextPutAll: 'Image: '; print: SystemVersion current; cr;
		nextPutAll: 'VM:    '; nextPutAll: Smalltalk vm version; cr.
	self quit.! !

TestCase subclass: #CommandLineTest
	instanceVariableNames: 'commandLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine-Tests'!

!CommandLineTest commentStamp: 'AdrienBarreau 2/12/2011 15:55' prior: 0!
That class tests the API given by AbstractUserInput. Since that class is obviously abstract, it can't be instantiated, the tests are running on CommandLine (which inherits from AbstractUserInput)!

!CommandLineTest class methodsFor: 'as yet unclassified' stamp: 'AdrienBarreau 2/12/2011 13:54'!
resources
	^ {CommandLineResource}! !

!CommandLineTest methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 14:10'!
commandLine
	^ commandLine! !

!CommandLineTest methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 14:10'!
parameters
	^ #('noOpt1' 'noOpt2.txt' 'noOpt3.avi' '-option1' 'opt11' 'opt12.txt' 'opt13.avi' '-option2' '-option3' 'opt31.st' 'opt32' '-option4')! !

!CommandLineTest methodsFor: 'running' stamp: 'CamilloBruni 5/1/2012 21:13'!
setUp
	commandLine := CommandLineArguments new.! !

!CommandLineTest methodsFor: 'tests' stamp: 'StephaneDucasse 3/14/2011 13:51'!
testAllArgumentAtOption
	"self debug: #testAllArgumentAtOption"
	
	self assert: (self commandLine allArgumentsAtOption: #option1) = #('opt11' 'opt12.txt' 'opt13.avi').
	self assert: (self commandLine allArgumentsAtOption: #option2) isNil.
	self assert: (self commandLine allArgumentsAtOption: #foo) isNil.
	self assert: (self commandLine allArgumentsAtOption: nil) isNil.! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 15:05'!
testAllArgumentsNoOption
	self assert: (self commandLine allArgumentsNoOption) = #('noOpt1' 'noOpt2.txt' 'noOpt3.avi').
	self assert: (self commandLine allArgumentsNoOption) = (self commandLine allArgumentsAtOption: #*)! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 14:48'!
testAllFileTyped
	self assert: (self commandLine allFilesTyped: #txt) = #('noOpt2.txt' 'opt12.txt').
	self assert: (self commandLine allFilesTyped: #foo) isNil.
	self assert: (self commandLine allFilesTyped: nil) isNil! !

!CommandLineTest methodsFor: 'tests' stamp: 'CamilloBruni 5/1/2012 14:10'!
testAllParameters
	self assert: (self commandLine allParameters = self parameters)! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 14:53'!
testFirstArgumentAtOption
	self assert: (self commandLine firstArgumentAtOption: #option1) = 'opt11'.
	self assert: (self commandLine firstArgumentAtOption: #option2) isNil.
	self assert: (self commandLine firstArgumentAtOption: #nil) isNil.! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 14:58'!
testHasFilesTyped
	self assert: (self commandLine hasFilesTyped: #txt).
	self deny: (self commandLine hasFilesTyped: #foo).
	self deny: (self commandLine hasFilesTyped: nil).! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 14:59'!
testHasOption
	self assert: (self commandLine hasOption: #option1).
	self assert: (self commandLine hasOption: #option4).
	self deny: (self commandLine hasOption: #foo).
	self deny: (self commandLine hasOption: nil).! !

!CommandLineTest methodsFor: 'tests' stamp: 'CamilloBruni 5/1/2012 20:50'!
testHasParameters
	self assert: (self commandLine hasArguments)! !

!CommandLineTest methodsFor: 'tests' stamp: 'AdrienBarreau 2/12/2011 15:02'!
testParameterAt
	self assert: (self commandLine parameterAt: 1) = 'noOpt1'.
	self assert: (self commandLine parameterAt: 4) = '-option1'.
	self deny: (self commandLine parameterAt: 5) = 'foo'.
	self deny: (self commandLine parameterAt: 1) isNil.
	self assert: (self commandLine parameterAt: nil) isNil.! !
DefaultCommandLineHandler initialize!
